/**
 * Copyright (C) 2008  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.orchestra.console.webui.novaview;

import java.util.Iterator;
import java.util.List;
import java.util.Map;

import org.exoplatform.webui.config.annotation.ComponentConfig;
import org.exoplatform.webui.core.UIComponent;
import org.exoplatform.webui.core.UIContainer;
import org.ow2.orchestra.console.webui.commons.UINovaGrid;

/**
 * UINovaView is the place where main data will be display. It can be view as a
 * panel or as a window.
 * 
 * @author Le Gall Rodrigue <rodrigue.le-gall@bull.net>
 * @author Elwalid BRAHIM <elwalid.brahim@bull.net>
 */

@ComponentConfig(template = "system:/groovy/novabpm/commons/webui/UINovaView.gtmpl")
public class UINovaView extends UIContainer {

  /**
   * Grid of data
   */
  private UINovaGrid grid;

  /**
   * Map of parameter <-> value that the view need do display result
   */
  private Map<String, String> params;

  /**
   * In the defaut case, a nova view is not in the detailed mode.
   */
  private boolean detail = false;

  /**
   * The display name of the nova view
   */
  private String displayName;

  /**
   * The type of the view
   */
  private String type = UINovaView.TYPE_FORM;

  public static final String TYPE_FORM = "Form";
  public static final String TYPE_PROCESS = "Processes";
  public static final String TYPE_INSTANCE = "Instances";
  public static final String TYPE_ACTIVITY = "Activities";

  /**
   * Constructor
   * 
   * @throws Exception
   **/
  public UINovaView() throws Exception {
    super();
    grid = addChild(UINovaGrid.class, null, null);
  }

  /**
   * Init method
   * 
   * @throws Exception
   **/
  public void init() throws Exception {
  }

  /**
   * Set that the nova view does not display it grid
   */
  public void setNoGrid() {
    grid.setRendered(false);
  }

  /**
   * Switch to detailed mode
   */
  public void setDetail() {
    this.detail = true;
    this.grid.setComponentConfig(UINovaGrid.class, "detail");
  }

  /**
   * set a specific configuration to the Grid
   * 
   * @param clazz the class where the config is defined
   * @id the id of the config
   */
  public void setGridConfig(Class< ? > clazz, String id) {
    this.grid.setComponentConfig(clazz, id);
  }

  /**
   * Check if the nova view is in detailed mode
   * 
   * @return boolean
   */
  public boolean isDetail() {
    return this.detail;
  }

  /**
   * Set the parameters of the nova view
   * 
   * @param parameters
   */
  public void setParams(Map<String, String> params) {
    this.params = params;
  }

  /**
   * Get the parameter value of the given parameter key
   * 
   * @param parameter's key
   * @return parameter value
   */
  public String getParam(String key) {
    if (this.params == null) {
      return null;
    }
    return this.params.get(key);
  }

  /**
   * Check if the given parameter exist
   * 
   * @param parameter's key
   * @return boolean
   */
  public boolean hasParam(String key) {
    if (this.params == null) {
      return false;
    }
    return this.params.containsKey(key);
  }

  /**
   * Get the grid of the nova view
   * 
   * @return the grid
   */
  public UINovaGrid getGrid() {
    return grid;
  }

  /**
   * Get the factory of views
   * 
   * @return nova view factory
   */
  public UINovaViewFactory getFactory() {
    UINovaViewContainer container = getAncestorOfType(UINovaViewContainer.class);
    return container.getFactory();
  }

  /**
   * The update method allow to update the data of the nova view and also to
   * propagate the update request to all the childs.
   * 
   * @throws Exception
   */
  public void update() throws Exception {
    updateData();
    List<UIComponent> children = getChildren();
    Iterator<UIComponent> ite = children.iterator();
    while (ite.hasNext()) {
      UIComponent uic = ite.next();
      if (UINovaView.class.isInstance(uic)) {
        UINovaView.class.cast(uic).update();
      } else if (UINovaViewContainer.class.isInstance(uic)) {
        UINovaViewContainer.class.cast(uic).update();
      }
    }
  }

  /**
   * This method allow to update the data of the nova view. It should be
   * overrided.
   * 
   * @throws Exception
   */
  public void updateData() throws Exception {
  }

  /**
   * Set the display name for the nova view
   * 
   * @param display name
   */
  public void setDisplayName(String displayName) {
    this.displayName = displayName;
  }

  /**
   * Get the display name of the nova view
   * 
   * @return display name
   */
  public String getDisplayName() {
    return this.displayName;
  }

  /**
   * get the type of the view
   * 
   * @return the type
   */
  public String getType() {
    return this.type;
  }

  /**
   * set the type of the view
   * 
   * @param type the type
   */
  public void setType(String type) {
    this.type = type;
  }
}
