/**
 * Copyright (C) 2008  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.orchestra.console.webui.novaview;

import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

import org.exoplatform.webui.Util;
import org.exoplatform.webui.application.ConfigurationManager;
import org.exoplatform.webui.application.WebuiApplication;
import org.exoplatform.webui.application.WebuiRequestContext;
import org.exoplatform.webui.config.Component;
import org.exoplatform.webui.config.annotation.ComponentConfig;
import org.exoplatform.webui.core.UIComponent;
import org.exoplatform.webui.core.UIContainer;
import org.exoplatform.webui.event.Event;
import org.exoplatform.webui.event.EventListener;

/**
 * UINovaViewManagement is responsible for showing the NovaView in the right
 * format : Popup or tab
 * 
 * @author Le Gall Rodrigue <rodrigue.le-gall@bull.net>
 * @author Elwalid BRAHIM <elwalid.brahim@bull.net>
 */
@ComponentConfig(template = "system:/groovy/novabpm/commons/webui/UINovaViewManagement.gtmpl")
public class UINovaViewManagement extends UIContainer {

  /**
   * The factory that allow to create views or containers
   */
  private UINovaViewFactory factory;

  /**
   * The viewer which is responsible of the rendering of views
   */
  private UINovaViewer viewer;

  /**
   * The history of showed views
   */
  private LinkedList<String> history;

  /** Contructor **/
  public UINovaViewManagement() throws Exception {
    super();
    history = new LinkedList<String>();
    viewer = createUIComponent(UINovaViewer.class, null, "viewer");
    viewer.setParent(this);
  }

  /**
   * Get the nova viewer
   * 
   * @return the nova viewer of the view manager
   */
  public UINovaViewer getViewer() {
    return viewer;
  }

  /**
   * This method render the viewer component
   * 
   * @throws Exception
   */
  public void renderViewer() throws Exception {
    WebuiRequestContext context = WebuiRequestContext.getCurrentInstance();
    viewer.processRender(context);
  }

  /**
   * Create a nova view factory with a given config id
   * 
   * @param config id
   * @throws Exception
   * @todo optimize this
   */
  public void setConfigFactory(String configId) throws Exception {
    WebuiRequestContext context = WebuiRequestContext.getCurrentInstance();
    WebuiApplication app = (WebuiApplication) context.getApplication();
    ConfigurationManager configManager = app.getConfigurationManager();

    UIComponent parent = (UIComponent) getParent();
    Component config = configManager.getComponentConfig(parent.getClass(), configId);
    this.factory = Util.createObject(UINovaViewFactory.class, config.getInitParams());
    this.factory.setComponentConfig("ViewFactory", config);
    config.getUIComponentLifecycle().init(this.factory, context);
    this.factory.setRendered(false);
    addChild(this.factory);
  }

  /**
   * Get the view factory
   * 
   * @return the nova view factory
   */
  public UINovaViewFactory getViewFactory() {
    return this.factory;
  }

  /**
   * Add a container of views
   * 
   * @param type of the container to create
   * @param config id of the view container
   * @param component id of the container
   * @return the created view container
   * @throws Exception
   */
  public UINovaViewContainer addViewContainer(Class< ? extends UINovaViewContainer> type, String configId, String componentId) 
    throws Exception {
    UINovaViewContainer view = this.factory.createViewContainer(type, configId, componentId);
    addChild(view);
    return view;
  }

  /**
   * Add a views container as view and link it as child to the given parent
   * component id
   * 
   * @param type of the container to create
   * @param config id
   * @param component id
   * @param display name
   * @param parent id
   * @return the created view container
   * @throws Exception
   */
  public UINovaViewContainer addViewContainerAsView(Class< ? extends UINovaViewContainer> type, 
                                                    String configId, 
                                                    String componentId, 
                                                    String displayName,
                                                    String parentId) throws Exception {
    UIContainer container = this.getChildById(parentId);
    if (container != null) {
      UINovaViewContainer view = container.getChildById(componentId);
      if (view == null) {
        view = this.factory.createViewContainer(type, configId, componentId);
        if (displayName != null) {
          view.setDisplayName(displayName);
        } else {
          view.setDisplayName(componentId);
        }
        container.addChild(view);
      }
      return view;
    }
    return null;
  }

  /**
   * Add a nova view to the container represented by the given parent id
   * 
   * @param type of the view to create
   * @param config id
   * @param component id
   * @param display name
   * @param parent id
   * @return the create nova view
   * @throws Exception
   */
  public UINovaView addView(Class< ? extends UINovaView> type, 
                            String configId, 
                            String componentId, 
                            String displayName, 
                            String parentId) 
    throws Exception {
    UIContainer container = this.getChildById(parentId);
    if (container != null) {
      UINovaView view = container.getChildById(componentId);
      if (view == null) {
        view = this.factory.createView(type, configId, componentId);
        if (displayName != null) {
          view.setDisplayName(displayName);
        } else {
          view.setDisplayName(componentId);
        }
        container.addChild(view);
        view.init();
        return view;
      }
      return view;
    }
    return null;
  }

  /**
   * Display the view represented by the given view id
   * 
   * @param view id
   * @throws Exception
   */
  public void showView(String id) throws Exception {
    this.history.remove(id);
    this.history.addFirst(id);
    List<UIComponent> list = getChildren();
    for (UIComponent child : list) {
      if (UINovaViewContainer.class.isInstance(child)) {
        UINovaViewContainer container = UINovaViewContainer.class.cast(child);
        container.switchView(id);
      }
    }
    updateView(id);
  }

  /**
   * Update the view represented by the given view id
   * 
   * @param view id
   * @throws Exception
   */
  public void updateView(String id) throws Exception {
    List<UIComponent> list = getChildren();
    for (UIComponent child : list) {
      if (UINovaViewContainer.class.isInstance(child)) {
        UINovaViewContainer container = UINovaViewContainer.class.cast(child);
        UIComponent c = container.getChildById(id);
        if (c != null) {
          if (UINovaView.class.isInstance(c)) {
            UINovaView.class.cast(c).update();
          } else if (UINovaViewContainer.class.isInstance(c)) {
            UINovaViewContainer.class.cast(c).update();
          }
        }
      }
    }
  }

  /**
   * Refresh the current displayed view
   * 
   * @throws Exception
   */
  public void reloadCurrentView() throws Exception {
    updateView(this.history.getFirst());
  }

  /**
   * Close the view represented by the given view id
   * 
   * @param view id
   * @throws Exception
   */
  public void closeView(String id) throws Exception {
    List<UIComponent> list = getChildren();
    for (UIComponent child : list) {
      if (UINovaViewContainer.class.isInstance(child)) {
        UINovaViewContainer container = UINovaViewContainer.class.cast(child);
        container.removeChildById(id);
      }
    }
    this.history.remove(id);
    // show the last opened view
    showView(this.history.getFirst());
  }

  /**
   * Check if the view, represented by the given view id, exist
   * 
   * @param view id
   * @return boolean
   */
  public boolean hasView(String id) {
    List<UIComponent> list = getChildren();
    for (UIComponent child : list) {
      if (UINovaViewContainer.class.isInstance(child)) {
        UINovaViewContainer container = UINovaViewContainer.class.cast(child);
        if (container.getChildById(id) != null) {
          return true;
        }
      }
    }
    return false;
  }

  /**
   * return the view, represented by the given view id
   * 
   * @param view id
   * @return UINovaView
   */
  public UINovaView getView(String id) {
    List<UIComponent> list = getChildren();
    for (UIComponent child : list) {
      if (UINovaViewContainer.class.isInstance(child)) {
        UINovaViewContainer container = UINovaViewContainer.class.cast(child);
        UIComponent c = container.getChildById(id);
        if (c != null) {
          if (UINovaView.class.isInstance(c)) {
            return UINovaView.class.cast(c);
          }
        }
      }
    }
    return null;
  }

  /**
   * Update by ajax the container, this will so update by ajax all the contained
   * views
   * 
   * @param request context
   */
  public void addContainerToUpdateByAjax(WebuiRequestContext requestContext) {
    List<UIComponent> list = getChildren();
    Iterator<UIComponent> ite = list.iterator();
    while (ite.hasNext()) {
      UIComponent child = ite.next();
      if (UINovaViewContainer.class.isInstance(child)) {
        if (UINovaViewContainer.class.cast(child).isSwitchView()) {
          requestContext.addUIComponentToUpdateByAjax(child);
        }
      }
    }
  }

  /**
   * This subclass is the listener of the view child request event. It get the
   * view id of the view request and display this view.
   */
  public static class ViewChildActionListener extends EventListener<UINovaViewContainer> {

    /**
     * The execute method of the listener
     * 
     * @throws Exception
     */
    public void execute(Event<UINovaViewContainer> event) throws Exception {
      UINovaViewContainer uiContainer = event.getSource();
      UINovaViewManagement viewMgt = uiContainer.getAncestorOfType(UINovaViewManagement.class);
      String id = event.getRequestContext().getRequestParameter(UIComponent.OBJECTID);
      viewMgt.showView(id);
      viewMgt.addContainerToUpdateByAjax(event.getRequestContext());
      event.getRequestContext().addUIComponentToUpdateByAjax(viewMgt.getViewer());
      //System.out.println("Container: "+uiContainer.getId()+" render view: "+id
      // +" via management:"+viewMgt.getId());
    }
  }

  /**
   * This subclass is the listener of the close view request event. It get the
   * view id of the close request and close this view.
   */
  public static class CloseViewActionListener extends EventListener<UINovaViewContainer> {

    /**
     * The execute method of the listener
     * 
     * @throws Exception
     */
    public void execute(Event<UINovaViewContainer> event) throws Exception {
      UINovaViewContainer uiContainer = event.getSource();
      UINovaViewManagement viewMgt = uiContainer.getAncestorOfType(UINovaViewManagement.class);
      String id = event.getRequestContext().getRequestParameter(UIComponent.OBJECTID);
      viewMgt.closeView(id);
      viewMgt.addContainerToUpdateByAjax(event.getRequestContext());
      event.getRequestContext().addUIComponentToUpdateByAjax(viewMgt.getViewer());
      // System.out.println("Container: "+uiContainer.getId()+" close view: "+id
      // +" via management:"+viewMgt.getId());
    }
  }
}
